----------------------------------------------------------------------------------
-- Company:
-- Engineer:
--
-- Create Date: 06/04/2018 11:06:32 AM
-- Design Name:
-- Module Name: FileOutput_v1_0_Sim - Behavioral
-- Project Name:
-- Target Devices:
-- Tool Versions:
-- Description:
--
-- Dependencies:
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.math_real.all;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity FileOutput_v1_0_Sim is
--  Port ( );
end FileOutput_v1_0_Sim;

architecture Behavioral of FileOutput_v1_0_Sim is
    constant C_DATA_IN_TDATA_WIDTH : integer := 32;

    component FileOutput_v1_0
    generic (
      C_DATA_IN_TDATA_WIDTH : integer := 32;
      OUT_FILE : string := "/tmp/testout.bin"
    );
    port (
      data_in_aclk    : in  std_logic;
      data_in_aresetn : in  std_logic;
      data_in_tready  : out std_logic;
      data_in_tdata   : in  std_logic_vector(C_DATA_IN_TDATA_WIDTH-1 downto 0);
      data_in_tstrb   : in  std_logic_vector((C_DATA_IN_TDATA_WIDTH/8)-1 downto 0);
      data_in_tlast   : in  std_logic;
      data_in_tvalid  : in  std_logic
    );
    end component FileOutput_v1_0;

    signal data_in_aclk    : std_logic := '0';
    signal data_in_aresetn : std_logic := '0';
    signal data_in_tready  : std_logic;
    signal data_in_tdata   : std_logic_vector(C_DATA_IN_TDATA_WIDTH-1 downto 0);
    signal data_in_tstrb   : std_logic_vector((C_DATA_IN_TDATA_WIDTH/8)-1 downto 0) := (others => '1');
    signal data_in_tlast   : std_logic;
    signal data_in_tvalid  : std_logic;


    type t_InputState is (IDLE, GENDATA, OFFSET, SENDING_DATA, DELAY, DONE);
    signal input_state : t_InputState;
    signal data_offset : natural;
    signal offset_counter : natural;
    signal data_delay : natural;
    signal delay_counter : natural;
    signal data_counter : natural;
    signal data_counter_max : natural;
begin
    FileOutput_v1_0_i : FileOutput_v1_0
    generic map (
      C_DATA_IN_TDATA_WIDTH => C_DATA_IN_TDATA_WIDTH
    )
    port map (
      data_in_aclk    => data_in_aclk,
      data_in_aresetn => data_in_aresetn,
      data_in_tready  => data_in_tready,
      data_in_tdata   => data_in_tdata,
      data_in_tstrb   => data_in_tstrb,
      data_in_tlast   => data_in_tlast,
      data_in_tvalid  => data_in_tvalid
    );


    data_in_aclk <= not data_in_aclk after 10 ns;
    data_in_aresetn <= '1' after 300 ns;

    RND_DATA_AUTOMATON : process(data_in_aclk)
      variable seed1, seed2: positive;               -- seed values for random generator
      variable rand: real;   -- random real-number value in range 0 to 1.0
      variable range_of_rand : real := 50.0;    -- the range of random values created will be 0 to +1000.
      variable range_of_rand_data : real := 1000.0;    -- the range of random values created will be 0 to +1000.
    begin
      if rising_edge(data_in_aclk) then
        if data_in_aresetn = '0' then
          input_state <= IDLE;
        else
          data_in_tlast <= '0';
          data_in_tvalid <= '0';

          case(input_state) is
            when IDLE =>
              data_counter <= 0;
              offset_counter <= 0;
              delay_counter <= 0;
              input_state <= GENDATA;
            when GENDATA =>
              input_state <= OFFSET;

              uniform(seed1, seed2, rand);   -- generate random number
              data_offset <= natural(rand*range_of_rand);  -- rescale to 0..1000, convert integer part

              uniform(seed1, seed2, rand);
              data_delay <= natural(rand*range_of_rand);

              uniform(seed1, seed2, rand);
              data_counter_max <= natural(rand*range_of_rand_data);
            when OFFSET =>
              if offset_counter = data_offset then
                input_state <= SENDING_DATA;
              end if;
              offset_counter <= offset_counter + 1;
            when SENDING_DATA =>
              data_in_tvalid <= '1';
              if data_counter = data_counter_max then
                input_state <= DELAY;
              end if;
              if data_counter = data_counter_max-1 then
                data_in_tlast <= '1';
              end if;
              if data_in_tready = '1' then
                data_counter <= data_counter +1;
              end if;
            when DELAY =>
              if delay_counter = data_delay then
                input_state <= DONE;
              end if;
              delay_counter <= delay_counter + 1;
            when DONE =>
              input_state <= IDLE;
          end case;
        end if;
      end if;
    end process; -- end RND_DATA_AUTOMATON

    GEN_INPUT : process(data_in_aclk)
      variable seed1, seed2: positive;               -- seed values for random generator
      variable rand: real;   -- random real-number value in range 0 to 1.0
      variable range_of_rand : real :=  4294967293.0;    -- the range of random values created will be 0 to +1000.
    begin
      if rising_edge(data_in_aclk) then
        if(data_in_aresetn = '0') then
          data_in_tdata <= (others => '0');
        else
          if(input_state = SENDING_DATA and data_in_tready = '1') then
            uniform(seed1, seed2, rand);
            data_in_tdata <= std_logic_vector(to_unsigned(natural(rand*range_of_rand), data_in_tdata'length));
          end if;
        end if;
      end if;
    end process; -- end GEN_INPUT

end Behavioral;
