----------------------------------------------------------------------------------
-- Company:
-- Engineer:
--
-- Create Date: 06/12/2018 01:52:08 PM
-- Design Name:
-- Module Name: LISCCalculator - Behavioral
-- Project Name:
-- Target Devices:
-- Tool Versions:
-- Description:
--
-- Dependencies:
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
----------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity LISCCalculator is
    Generic(
      C_OUTPUT_WIDTH : natural := 32;
      C_INPUT_WIDTH : natural := 16
    );
    Port ( clk : in STD_LOGIC;
           rst : in STD_LOGIC;
           ce : in STD_LOGIC;
           input_number : in STD_LOGIC_VECTOR (C_INPUT_WIDTH-1 downto 0);
           start : in STD_LOGIC;
           output_lisc : out STD_LOGIC_VECTOR (C_OUTPUT_WIDTH-1 downto 0);
           output_valid : out STD_LOGIC_VECTOR (C_OUTPUT_WIDTH/8-1 downto 0);
           done : out STD_LOGIC);
end LISCCalculator;

architecture Behavioral of LISCCalculator is
  type FSM_TYPE is (S_IDLE, S_CALCULATE);
  signal current_state : FSM_TYPE;
  signal next_state : FSM_TYPE;

  signal last_code : boolean;
  signal current_number : unsigned(C_INPUT_WIDTH-1 downto 0);
  signal current_progress : natural;
begin


FSM_SEQ : process(clk)
begin
  if rising_edge(clk) then
    if rst = '1' then
      current_state <= S_IDLE;
    elsif(ce = '1') then
      current_state <= next_state;

      case( current_state ) is
        when S_IDLE =>
          if next_state = S_CALCULATE then
            current_number <= unsigned(input_number);
          end if;
        when S_CALCULATE =>
          current_number <= current_number-C_OUTPUT_WIDTH/8*255;
        when others =>
      end case; -- end case current_state
    end if;
  end if;
end process; -- end FSM_SEQ

FSM_NEXT_COMB : process(current_state, start, last_code)
begin
  case( current_state ) is
    when S_IDLE =>
      if start = '1' then
        next_state <= S_CALCULATE;
      end if;
    when S_CALCULATE =>
      if last_code = true then
        next_state <= S_IDLE;
      end if;
    when others =>
      next_state <= next_state;
  end case; -- end case current_state
end process; -- end FSM_NEXT_COMB

FSM_OUT_COMP : process(current_state, current_number)
  variable tmp : integer;
  variable curLimit : natural;
begin
  last_code <= false;
  output_valid <= (others => '0');
  output_lisc <= (others => '0');
  done <= '0';
  case( current_state ) is
    when S_CALCULATE =>
      for i in 0 to C_OUTPUT_WIDTH/8-1 loop
        curLimit := (i+1)*255;
        output_valid(i) <= '1';
        if curLimit <= current_number then
          output_lisc((i+1)*8-1 downto i*8) <= (others => '1');
        else
          tmp := to_integer(current_number) - i*255;
          output_lisc((i+1)*8-1 downto i*8) <= std_logic_vector(to_unsigned(tmp, 8));
          last_code <= true;
          done <= '1';
          exit;
        end if;
      end loop;
    when others =>

  end case; -- end case current_state
end process; -- end FSM_OUT_COMP

end Behavioral;
