library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.LZ4CompressorLib.all;
use ieee.std_logic_misc.all;

entity LZ4Compressor_v0_1 is
	generic (
		-- Users to add parameters here

		-- User parameters ends
		-- Do not modify the parameters beyond this line


		-- Parameters of Axi Slave Bus Interface IN_DATA_AXI
		C_IN_DATA_AXI_TDATA_WIDTH	: integer	:= 128;

		-- Parameters of Axi Master Bus Interface OUT_DATA_AXI
		C_OUT_DATA_AXI_TDATA_WIDTH	: integer	:= 32;
		C_OUT_DATA_AXI_START_COUNT	: integer	:= 32;

		-- Parameters of Axi Slave Bus Interface IN_CTRL_AXI
		C_IN_CTRL_AXI_DATA_WIDTH	: integer	:= 32;
		C_IN_CTRL_AXI_ADDR_WIDTH	: integer	:= 4
	);
	port (
		-- Users to add ports here

		-- User ports ends
		-- Do not modify the ports beyond this line


		-- Ports of Axi Slave Bus Interface IN_DATA_AXI
		in_data_axi_aclk	: in std_logic;
		in_data_axi_aresetn	: in std_logic;
		in_data_axi_tready	: out std_logic;
		in_data_axi_tdata	: in std_logic_vector(C_IN_DATA_AXI_TDATA_WIDTH-1 downto 0);
		in_data_axi_tstrb	: in std_logic_vector((C_IN_DATA_AXI_TDATA_WIDTH/8)-1 downto 0);
		in_data_axi_tlast	: in std_logic;
		in_data_axi_tvalid	: in std_logic;

		-- Ports of Axi Master Bus Interface OUT_DATA_AXI
		out_data_axi_aclk	: in std_logic;
		out_data_axi_aresetn	: in std_logic;
		out_data_axi_tvalid	: out std_logic;
		out_data_axi_tdata	: out std_logic_vector(C_OUT_DATA_AXI_TDATA_WIDTH-1 downto 0);
		out_data_axi_tstrb	: out std_logic_vector((C_OUT_DATA_AXI_TDATA_WIDTH/8)-1 downto 0);
		out_data_axi_tlast	: out std_logic;
		out_data_axi_tready	: in std_logic;

		-- Ports of Axi Slave Bus Interface IN_CTRL_AXI
		in_ctrl_axi_aclk	: in std_logic;
		in_ctrl_axi_aresetn	: in std_logic;
		in_ctrl_axi_awaddr	: in std_logic_vector(C_IN_CTRL_AXI_ADDR_WIDTH-1 downto 0);
		in_ctrl_axi_awprot	: in std_logic_vector(2 downto 0);
		in_ctrl_axi_awvalid	: in std_logic;
		in_ctrl_axi_awready	: out std_logic;
		in_ctrl_axi_wdata	: in std_logic_vector(C_IN_CTRL_AXI_DATA_WIDTH-1 downto 0);
		in_ctrl_axi_wstrb	: in std_logic_vector((C_IN_CTRL_AXI_DATA_WIDTH/8)-1 downto 0);
		in_ctrl_axi_wvalid	: in std_logic;
		in_ctrl_axi_wready	: out std_logic;
		in_ctrl_axi_bresp	: out std_logic_vector(1 downto 0);
		in_ctrl_axi_bvalid	: out std_logic;
		in_ctrl_axi_bready	: in std_logic;
		in_ctrl_axi_araddr	: in std_logic_vector(C_IN_CTRL_AXI_ADDR_WIDTH-1 downto 0);
		in_ctrl_axi_arprot	: in std_logic_vector(2 downto 0);
		in_ctrl_axi_arvalid	: in std_logic;
		in_ctrl_axi_arready	: out std_logic;
		in_ctrl_axi_rdata	: out std_logic_vector(C_IN_CTRL_AXI_DATA_WIDTH-1 downto 0);
		in_ctrl_axi_rresp	: out std_logic_vector(1 downto 0);
		in_ctrl_axi_rvalid	: out std_logic;
		in_ctrl_axi_rready	: in std_logic
	);
end LZ4Compressor_v0_1;
architecture arch_imp of LZ4Compressor_v0_1 is

	-- component declaration
	component LZ4Compressor_v0_1_IN_DATA_AXI is
		generic (
		C_HASHNATOR_COUNT : natural := 8;
		C_HASHNATOR_WIDTH : natural := 32;
		C_HASHNATOR_OFFSET : natural := 8;
		C_S_AXIS_TDATA_WIDTH	: integer	:= 32
		);
		port (
		UNIT_WORKING : in std_logic;
		MEM_MODE : in std_logic;
		MEM_MF_DATA_1 : out std_logic_vector((C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET) -1 downto 0);
		MEM_MF_DATA_2 : out std_logic_vector((C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET) -1 downto 0);
		MEM_MF_ADDR_1 : in LZ4Address;
		MEM_MF_ADDR_2 : in LZ4Address;

		MEM_ADDR : in LZ4Address;
		MEM_HASHING_DATA : out std_logic_vector((C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET) -1 downto 0);
		MEM_STREAM_DATA : out  std_logic_vector((C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET) -1 downto 0);
		MEM_STREAM_DATA_VALID : out std_logic_vector((C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET)/8 -1 downto 0);


		MEM_OUT_DATA_VALID : out std_logic_vector((C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET)/8 -1 downto 0);
		MEM_OUT_DATA : out std_logic_vector((C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET) -1 downto 0);
		MEM_OUT_ADDR : in LZ4Address;
		MEM_LENGTH : out LZ4Address;


		DATA_READY : out std_logic;
		DATA_EMPTY : out std_logic;
		DATA_ENDING : out std_logic;


		S_AXIS_ACLK	: in std_logic;
		S_AXIS_ARESETN	: in std_logic;
		S_AXIS_TREADY	: out std_logic;
		S_AXIS_TDATA	: in std_logic_vector(C_S_AXIS_TDATA_WIDTH-1 downto 0);
		S_AXIS_TSTRB	: in std_logic_vector((C_S_AXIS_TDATA_WIDTH/8)-1 downto 0);
		S_AXIS_TLAST	: in std_logic;
		S_AXIS_TVALID	: in std_logic
		);
	end component LZ4Compressor_v0_1_IN_DATA_AXI;

	component LZ4Compressor_v0_1_OUT_DATA_AXI
		generic (
		C_M_AXIS_TDATA_WIDTH : integer := 32;
		C_FIFO_LENGTH        : natural := 11;
		C_DATA_WIDTH         : natural := 88;
		C_BLOCK_LENGTH       : natural := 5
		);
		port (
		LIT_DATA       : in  std_logic_vector(C_DATA_WIDTH-1 downto 0);
		LIT_DATA_VALID : in std_logic_vector(C_DATA_WIDTH/8-1 downto 0);
		LIT_DATA_VALID_FLAG : in  std_logic;
		LIT_DATA_LAST  : in  std_logic;
		LIT_LENGTH     : in  LZ4MatchLength;
		MATCH_LENGTH   : in  LZ4MatchLength;
		MATCH_OFFSET   : in  LZ4Address;
		LENGTH_VALID   : in  std_logic;
		COMPRESSION_END : out std_logic;
		OUTPUT_READY   : out std_logic;
		M_PROC_READY   : out std_logic;
		M_AXIS_ACLK    : in  std_logic;
		M_AXIS_ARESETN : in  std_logic;
		M_AXIS_TVALID  : out std_logic;
		M_AXIS_TDATA   : out std_logic_vector(C_M_AXIS_TDATA_WIDTH-1 downto 0);
		M_AXIS_TSTRB   : out std_logic_vector((C_M_AXIS_TDATA_WIDTH/8)-1 downto 0);
		M_AXIS_TLAST   : out std_logic;
		M_AXIS_TREADY  : in  std_logic
		);
	end component LZ4Compressor_v0_1_OUT_DATA_AXI;

	component LZ4Compressor_v0_1_IN_CTRL_AXI is
		generic (
		C_S_AXI_DATA_WIDTH	: integer	:= 32;
		C_S_AXI_ADDR_WIDTH	: integer	:= 4
		);
		port (
		S_AXI_ACLK	: in std_logic;
		S_AXI_ARESETN	: in std_logic;
		S_AXI_AWADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		S_AXI_AWPROT	: in std_logic_vector(2 downto 0);
		S_AXI_AWVALID	: in std_logic;
		S_AXI_AWREADY	: out std_logic;
		S_AXI_WDATA	: in std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		S_AXI_WSTRB	: in std_logic_vector((C_S_AXI_DATA_WIDTH/8)-1 downto 0);
		S_AXI_WVALID	: in std_logic;
		S_AXI_WREADY	: out std_logic;
		S_AXI_BRESP	: out std_logic_vector(1 downto 0);
		S_AXI_BVALID	: out std_logic;
		S_AXI_BREADY	: in std_logic;
		S_AXI_ARADDR	: in std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
		S_AXI_ARPROT	: in std_logic_vector(2 downto 0);
		S_AXI_ARVALID	: in std_logic;
		S_AXI_ARREADY	: out std_logic;
		S_AXI_RDATA	: out std_logic_vector(C_S_AXI_DATA_WIDTH-1 downto 0);
		S_AXI_RRESP	: out std_logic_vector(1 downto 0);
		S_AXI_RVALID	: out std_logic;
		S_AXI_RREADY	: in std_logic
		);
	end component LZ4Compressor_v0_1_IN_CTRL_AXI;
	component PPSPacker
	generic (
	  PORTS      : natural := 8;
	  DATA_WIDTH : natural := 64
	);
	port (
	  clk         : in  std_logic;
	  data_in_we  : in  std_logic_vector(PORTS-1 downto 0);
	  data_in     : in  std_logic_vector(DATA_WIDTH*PORTS-1 downto 0);
	  data_out    : out std_logic_vector(DATA_WIDTH*PORTS-1 downto 0);
	  data_out_we : out std_logic_vector(PORTS-1 downto 0)
	);
	end component PPSPacker;

	component MatchProcessor
	generic (
	  C_MATCH_COUNT : natural := 8;
		C_MEMORY_BUS_WIDTH : natural := 128
	);
	port (
	  clk                  : in  std_logic;
	  rst                  : in  std_logic;
    out_ce      : in std_logic;
    matcher_done : in std_logic;
    memory_length : in LZ4Address;
	  match_valid          : in  std_logic_vector(C_MATCH_COUNT-1 downto 0);
	  match_address        : in  std_logic_vector(C_MATCH_COUNT*LZ4Address'length-1 downto 0);
	  match_source_address : in  std_logic_vector(C_MATCH_COUNT*LZ4Address'length-1 downto 0);
		finder_mf_data_1 : in std_logic_vector(C_MEMORY_BUS_WIDTH-1 downto 0);
		finder_mf_data_2 : in std_logic_vector(C_MEMORY_BUS_WIDTH-1 downto 0);
		finder_mf_address_1 : out LZ4Address;
		finder_mf_address_2 : out LZ4Address;
		out_literal_address : out LZ4Address;
		out_litetal_data_valid : out std_logic_vector(C_MEMORY_BUS_WIDTH/8-1 downto 0);
		out_literal_length : out LZ4MatchLength;
		out_match_length : out LZ4MatchLength;
		out_match_offset : out LZ4Address;
		out_match_valid : out std_logic;
		last_serve : out std_logic;
    out_match_ready : in std_logic
	);
	end component MatchProcessor;


	component ShiftRegister
	generic (
	  regDepth  : natural := 6;
	  dataWidth : natural := 32
	);
	port (
	  clk  : in  STD_LOGIC;
	  rst  : in  STD_LOGIC;
	  ce   : in  STD_LOGIC;
	  din  : in  STD_LOGIC_VECTOR (dataWidth-1 downto 0);
	  dout : out STD_LOGIC_VECTOR (dataWidth-1 downto 0)
	);
	end component ShiftRegister;

	constant C_HASHNATOR_COUNT  : natural := 8;
	constant C_HASHNATOR_WIDTH  : natural := 32;
	constant C_HASHNATOR_OFFSET : natural := 8;
	constant C_HASH_MEM_ADDR : natural := 8;
	constant C_FIFO_PORT_WIDTH  : natural := 32+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET;

	component LZ4CompressorController
	generic (
    MEMORY_BUS_STEP : natural := 8
	);
	port (
	  clk                 : in  std_logic;
	  rst                 : in  std_logic;
		ce 									: in std_logic;
	  data_ready          : in  std_logic;
    data_empty 					: in std_logic;
		data_ending					: in std_logic;
	  matcher_ce          : out std_logic;
	  delay_matcher_ce    : out std_logic;
	  finder_list_match   : out std_logic;
	  finder_match_addr   : out LZ4Address;
	  current_stream_head : out LZ4Address;
		literal_valid 		  : out std_logic;
	  mem_mode            : out std_logic;
	  mem_addr            : out LZ4Address;
	  unit_working        : out std_logic;
	  matcher_match       : in  std_logic_vector(MEMORY_BUS_STEP-1 downto 0);
	  matcher_match_addr  : in  std_logic_vector(MEMORY_BUS_STEP*LZ4Address'length-1 downto 0);
    matcher_match_source_addr : in std_logic_vector(MEMORY_BUS_STEP*LZ4Address'length-1 downto 0);
    matcher_rst_pipe 		: out std_logic;
	  finder_done         : in  std_logic;
	  finder_match_length : in  LZ4MatchLength;
    finder_current_end_stop : out LZ4Address;
    output_match_length : out LZ4Address;
    output_literal_length : out LZ4Address;
    output_match_offset : out LZ4Address;
    output_valid : out std_logic;
    pipe_override : out std_logic;
    pipe_override_valid : out std_logic_vector(MEMORY_BUS_STEP-1 downto 0)
	);
	end component LZ4CompressorController;

	component LZ4MatcherUnit is
        Generic(
               hashNators : natural := 8;
               hashNatorsOffset : natural := 8;
               dataWidth : natural := 32;
               addrWidth : natural := 16;
               hashMemAddrWidth : natural := 8
        );
        Port ( clk : in STD_LOGIC;
               rst : in STD_LOGIC;
               ce : in STD_LOGIC;
               data : in STD_LOGIC_VECTOR ((32+(8-1)*8)-1 downto 0);
               addr : in STD_LOGIC_VECTOR (16-1 downto 0);
               match : out STD_LOGIC_VECTOR(8-1 downto 0);
               match_addr : out STD_LOGIC_VECTOR (8*16-1 downto 0);
               match_source_addr : out STD_LOGIC_VECTOR (8*16-1 downto 0);
           		 rst_pipe : in STD_LOGIC);
    end component LZ4MatcherUnit;



	constant C_MEMORY_BUS_WIDTH : natural := (C_HASHNATOR_WIDTH+(C_HASHNATOR_COUNT-1)*C_HASHNATOR_OFFSET);
	subtype MEMORY_BUS is std_logic_vector(C_MEMORY_BUS_WIDTH -1 downto 0);
	subtype MEMORY_BUS_VALID is std_logic_vector(C_MEMORY_BUS_WIDTH/8 -1 downto 0);

	constant C_LITERAL_BUS_WIDTH : natural := 128;

	signal UNIT_WORKING          : std_logic;
	signal MEM_MODE          : std_logic;

	signal MEM_MF_ADDR_1        : LZ4Address;
	signal MEM_MF_ADDR_2        : LZ4Address;
	signal MEM_MF_DATA_1 : MEMORY_BUS;
	signal MEM_MF_DATA_2 : MEMORY_BUS;

  signal MEM_ADDR        : LZ4Address;
	signal MEM_ADDR_DELAY        : STD_LOGIC_VECTOR (LZ4Address'length-1 downto 0);
	signal MEM_HASHING_DATA : MEMORY_BUS;
	signal MEM_STREAM_DATA :  MEMORY_BUS;
	signal MEM_STREAM_DATA_VALID :  MEMORY_BUS_VALID;
	signal MEM_LENGTH : LZ4Address;

	signal PIPE_OVERRIDE : std_logic;
	signal PIPE_OVERRIDE_VALID : std_logic_vector(C_HASHNATOR_COUNT-1 downto 0);

	signal DATA_READY            : std_logic;
	signal DATA_EMPTY            : std_logic;
	signal DATA_ENDING 					: std_logic;
	signal DATA_EMPTY_DELAY            : std_logic;

	signal MATCH_DATA : MEMORY_BUS;
	signal MATCH_DATA_VALID : MEMORY_BUS_VALID;

	signal mf_list_match      : std_logic;
	signal mf_match_address   : LZ4Address;
	signal mf_current_address : LZ4Address;
	signal mf_match_length    : LZ4MatchLength;
	signal mf_length_done     : std_logic;
	signal mf_current_end_stop : LZ4Address;

	signal ctrl_matcher_ce          : std_logic;
	signal ctrl_delay_matcher_ce    : std_logic;
	signal ctrl_ce : std_logic;
	signal ctrl_lit_data_valid_flag : std_logic;
	signal compress_block_ce : std_logic;

	signal matcher_ce          : std_logic;
	signal delay_matcher_ce    : std_logic;
	signal match_length_finder_ce :std_logic;
	signal matcher_match       : std_logic_vector(7 downto 0);
	signal matcher_rst_pipe       : std_logic;
	signal matcher_match_addr  : std_logic_vector(LZ4Address'length*8-1 downto 0);
	signal matcher_match_source_addr : std_logic_vector(LZ4Address'length*8-1 downto 0);

	signal addr          : STD_LOGIC_VECTOR (LZ4Address'length-1 downto 0);
	signal match_addr    : STD_LOGIC_VECTOR (C_HASHNATOR_COUNT*LZ4Address'length-1 downto 0);
	signal match_source_addr    : STD_LOGIC_VECTOR (C_HASHNATOR_COUNT*LZ4Address'length-1 downto 0);

	signal LIT_DATA       : std_logic_vector(C_FIFO_PORT_WIDTH-1 downto 0);
	signal LIT_DATA_VALID : std_logic_vector(C_FIFO_PORT_WIDTH/8-1 downto 0);
	signal LIT_DATA_VALID_FLAG : std_logic;
	signal LIT_LENGTH     : LZ4MatchLength;
	signal LIT_DATA_LAST  : std_logic;
	signal MATCH_LENGTH   : LZ4MatchLength;
	signal MATCH_OFFSET   : LZ4Address;
	signal LENGTH_VALID   : std_logic;
	signal OUTPUT_READY   : std_logic;
	signal M_PROC_READY   : std_logic;
	signal COMPRESSION_END   : std_logic;

	signal pps_match_addr : std_logic_vector(C_HASHNATOR_COUNT*LZ4Address'length-1 downto 0);
	signal pps_match_source_addr : std_logic_vector(C_HASHNATOR_COUNT*LZ4Address'length-1 downto 0);
	signal pps_matcher_match : std_logic_vector(C_HASHNATOR_COUNT-1 downto 0);

	signal MEM_OUT_DATA_VALID : MEMORY_BUS_VALID;
	signal MEM_OUT_DATA : MEMORY_BUS;
	signal MEM_OUT_DATA_OFF : MEMORY_BUS;
	signal MEM_OUT_ADDR : LZ4Address;

	signal mpc_literal_address    : LZ4Address;
	signal mpc_litetal_data_valid : std_logic_vector(C_LITERAL_BUS_WIDTH/8-1 downto 0);
	signal mpc_literal_length     : LZ4MatchLength;
	signal mpc_match_length       : LZ4MatchLength;
	signal mpc_match_offset       : LZ4Address;
	signal mpc_match_valid        : std_logic;
	signal last_serve : std_logic;
	signal rst : std_logic;
	signal rstn : std_logic;

	signal work_clock_register : unsigned(63 downto 0) := (others => '0');
	signal data_clock_register : unsigned(63 downto 0) := (others => '0');

	signal match_processor_pipeline_ce : std_logic;
begin
	match_processor_pipeline_ce <= M_PROC_READY and OUTPUT_READY;

	END_SIMULATION_PROC : process
	begin
			wait until in_data_axi_tlast = '1';
			wait until COMPRESSION_END = '1';
			wait until out_data_axi_tlast = '1';
			wait for 200 ns;
			assert false report "Simulation Finished" severity failure;
	end process; -- end END_SIMULATION_PROC

	matcher_rst_pipe <= rst;

	matcher_ce <= ctrl_delay_matcher_ce and compress_block_ce;
	match_length_finder_ce <= compress_block_ce;
	ctrl_ce <= compress_block_ce;

	compress_block_ce <= '1';
	--compress_block_ce <= '1' when (M_PROC_READY or not LENGTH_VALID) else '0';

	LIT_DATA_VALID_FLAG <= compress_block_ce and ctrl_lit_data_valid_flag;

	matcher_match_addr <= match_addr;
	matcher_match_source_addr <= match_source_addr;

	-- Instantiation of Axi Bus Interface IN_DATA_AXI
	LZ4Compressor_v0_1_IN_DATA_AXI_inst : LZ4Compressor_v0_1_IN_DATA_AXI
	generic map (
		C_HASHNATOR_COUNT => C_HASHNATOR_COUNT,
		C_HASHNATOR_WIDTH => C_HASHNATOR_WIDTH,
		C_HASHNATOR_OFFSET => C_HASHNATOR_OFFSET,
		C_S_AXIS_TDATA_WIDTH	=> C_IN_DATA_AXI_TDATA_WIDTH
	)
	port map (
		UNIT_WORKING 	=> UNIT_WORKING,
		MEM_MODE 	=> MEM_MODE,
		MEM_MF_DATA_1 => MEM_MF_DATA_1,
		MEM_MF_DATA_2 => MEM_MF_DATA_2,
		MEM_MF_ADDR_1 	=> MEM_MF_ADDR_1,
		MEM_MF_ADDR_2 	=> MEM_MF_ADDR_2,
		MEM_ADDR 	=> MEM_ADDR,
		MEM_LENGTH => MEM_LENGTH,
		MEM_HASHING_DATA 	=> MEM_HASHING_DATA,
		MEM_STREAM_DATA 	=> MEM_STREAM_DATA,
		MEM_STREAM_DATA_VALID => MEM_STREAM_DATA_VALID,
		MEM_OUT_DATA_VALID => MEM_OUT_DATA_VALID,
		MEM_OUT_DATA => MEM_OUT_DATA,
		MEM_OUT_ADDR => MEM_OUT_ADDR,
		DATA_READY	=> DATA_READY,
		DATA_EMPTY  => DATA_EMPTY,
		DATA_ENDING => DATA_ENDING,
		S_AXIS_ACLK	=> in_data_axi_aclk,
		S_AXIS_ARESETN	=> rstn,
		S_AXIS_TREADY	=> in_data_axi_tready,
		S_AXIS_TDATA	=> in_data_axi_tdata,
		S_AXIS_TSTRB	=> in_data_axi_tstrb,
		S_AXIS_TLAST	=> in_data_axi_tlast,
		S_AXIS_TVALID	=> in_data_axi_tvalid
	);

	-- Instantiation of Axi Bus Interface OUT_DATA_AXI
	LZ4Compressor_v0_1_OUT_DATA_AXI_i : LZ4Compressor_v0_1_OUT_DATA_AXI
	generic map (
  	C_M_AXIS_TDATA_WIDTH => C_OUT_DATA_AXI_TDATA_WIDTH,
  	C_FIFO_LENGTH        => 15,
  	C_DATA_WIDTH         => C_LITERAL_BUS_WIDTH,
  	C_BLOCK_LENGTH       => 8
		)
	port map (
  	LIT_DATA => std_logic_vector(resize(unsigned(MEM_OUT_DATA_OFF), C_LITERAL_BUS_WIDTH)),
		LIT_DATA_VALID => mpc_litetal_data_valid,
  	LIT_DATA_VALID_FLAG => or_reduce(mpc_litetal_data_valid),
  	LIT_LENGTH     => mpc_literal_length,
		LIT_DATA_LAST  => DATA_EMPTY_DELAY,
  	MATCH_LENGTH   => mpc_match_length,
  	MATCH_OFFSET   => mpc_match_offset,
  	LENGTH_VALID   => mpc_match_valid,
		COMPRESSION_END => COMPRESSION_END,
		OUTPUT_READY   => open,
		M_PROC_READY   => open,
  	--OUTPUT_READY   => OUTPUT_READY,
		--M_PROC_READY   => M_PROC_READY,
  	M_AXIS_ACLK    => out_data_axi_aclk,
  	M_AXIS_ARESETN => rstn,
  	M_AXIS_TVALID  => out_data_axi_tvalid,
  	M_AXIS_TDATA   => out_data_axi_tdata,
  	M_AXIS_TSTRB   => out_data_axi_tstrb,
  	M_AXIS_TLAST   => out_data_axi_tlast,
  	M_AXIS_TREADY  => out_data_axi_tready
		);
	M_PROC_READY <= '1';
	OUTPUT_READY <= '1';

	-- Instantiation of Axi Bus Interface IN_CTRL_AXI
	LZ4Compressor_v0_1_IN_CTRL_AXI_inst : LZ4Compressor_v0_1_IN_CTRL_AXI
	generic map (
		C_S_AXI_DATA_WIDTH	=> C_IN_CTRL_AXI_DATA_WIDTH,
		C_S_AXI_ADDR_WIDTH	=> C_IN_CTRL_AXI_ADDR_WIDTH
	)
	port map (
		S_AXI_ACLK	=> in_ctrl_axi_aclk,
		S_AXI_ARESETN	=> in_ctrl_axi_aresetn,
		S_AXI_AWADDR	=> in_ctrl_axi_awaddr,
		S_AXI_AWPROT	=> in_ctrl_axi_awprot,
		S_AXI_AWVALID	=> in_ctrl_axi_awvalid,
		S_AXI_AWREADY	=> in_ctrl_axi_awready,
		S_AXI_WDATA	=> in_ctrl_axi_wdata,
		S_AXI_WSTRB	=> in_ctrl_axi_wstrb,
		S_AXI_WVALID	=> in_ctrl_axi_wvalid,
		S_AXI_WREADY	=> in_ctrl_axi_wready,
		S_AXI_BRESP	=> in_ctrl_axi_bresp,
		S_AXI_BVALID	=> in_ctrl_axi_bvalid,
		S_AXI_BREADY	=> in_ctrl_axi_bready,
		S_AXI_ARADDR	=> in_ctrl_axi_araddr,
		S_AXI_ARPROT	=> in_ctrl_axi_arprot,
		S_AXI_ARVALID	=> in_ctrl_axi_arvalid,
		S_AXI_ARREADY	=> in_ctrl_axi_arready,
		S_AXI_RDATA	=> in_ctrl_axi_rdata,
		S_AXI_RRESP	=> in_ctrl_axi_rresp,
		S_AXI_RVALID	=> in_ctrl_axi_rvalid,
		S_AXI_RREADY	=> in_ctrl_axi_rready
	);

	-- Add user logic here
	Delay_Matcher_i : ShiftRegister
	generic map (
	  regDepth  => 1,
	  dataWidth => C_MEMORY_BUS_WIDTH
	)
	port map (
	  clk  => in_data_axi_aclk,
	  rst  => rst,
	  ce   => match_processor_pipeline_ce,
	  din  => MEM_OUT_DATA, --Todo ReadEnable na tomhle memory interface
	  dout => MEM_OUT_DATA_OFF
	);


	Delay_Matcher_valid_i : ShiftRegister
	generic map (
		regDepth  => 1,
		dataWidth => C_MEMORY_BUS_WIDTH/8
	)
	port map (
		clk  => in_data_axi_aclk,
		rst  => matcher_rst_pipe,
		ce   => match_processor_pipeline_ce,
		din  => MEM_STREAM_DATA_VALID,
		dout => MATCH_DATA_VALID
	);

	PIPE_OVERRIDE_PROC : process(MATCH_DATA, MATCH_DATA_VALID, PIPE_OVERRIDE_VALID, MEM_STREAM_DATA_VALID)
	begin
		if(PIPE_OVERRIDE = '0') then
			LIT_DATA <= MATCH_DATA;
			LIT_DATA_VALID <= MATCH_DATA_VALID;
		else
			LIT_DATA <= MEM_STREAM_DATA;
			LIT_DATA_VALID <= (others => '0');
			LIT_DATA_VALID(PIPE_OVERRIDE_VALID'length-1 downto 0) <= (PIPE_OVERRIDE_VALID and MEM_STREAM_DATA_VALID(PIPE_OVERRIDE_VALID'length-1 downto 0));
		end if;
	end process; -- end PIPE_OVERRIDE_PROC


	Delay_Matcher_Empty_i : ShiftRegister
	generic map (
	  regDepth  => 1,
	  dataWidth => 1
	)
	port map (
	  clk  => in_data_axi_aclk,
	  rst  => matcher_rst_pipe,
	  ce   => match_processor_pipeline_ce,
	  din(0)  => DATA_EMPTY and last_serve,
	  dout(0) => DATA_EMPTY_DELAY
	);
	LIT_DATA_LAST <= DATA_EMPTY_DELAY;

	DELAY_MEM : process(in_data_axi_aclk)
	begin
		if rising_edge(in_data_axi_aclk) then
			if rst = '1' then
				MEM_ADDR_DELAY <= (others => '0');
			else
				MEM_ADDR_DELAY <= std_logic_vector(MEM_ADDR);
			end if;
		end if;
	end process; -- end DELAY_MEM

	HASH_MEM_ADDR : process(in_data_axi_aclk)
	begin
		if rising_edge(in_data_axi_aclk) then
			if rst = '1' then
				MEM_ADDR <= (others => '0');
				UNIT_WORKING <= '0';
				ctrl_matcher_ce <= '0';
			else
				ctrl_matcher_ce <= '0';
				if(DATA_READY = '1' and DATA_ENDING = '0') then
					UNIT_WORKING <= '1';
					ctrl_matcher_ce <= '1';
					MEM_ADDR <= MEM_ADDR + C_HASHNATOR_COUNT;
				end if;
			end if;
		end if;
	end process; -- end HASH_MEM_ADDR


	Delay_Matcher_Ce_i : ShiftRegister
	generic map (
	  regDepth  => 1,
	  dataWidth => 1
	)
	port map (
	  clk  => in_data_axi_aclk,
	  rst  => matcher_rst_pipe,
	  ce   => '1',
	  din(0)  => ctrl_matcher_ce,
	  dout(0) => ctrl_delay_matcher_ce
	);

	--LZ4CompressorController_i : LZ4CompressorController
	--generic map (
	--  MEMORY_BUS_STEP => C_HASHNATOR_COUNT
	--)
	--port map (
	--  clk                 => in_data_axi_aclk,
	--  rst                 => rst,
	--	ce 									=> ctrl_ce,
	--  data_ready          => DATA_READY,
	--	data_empty					=> DATA_EMPTY,
	--	data_ending 			  => DATA_ENDING,
	--  matcher_ce          => ctrl_matcher_ce,
	--  delay_matcher_ce    => ctrl_delay_matcher_ce,
	--  finder_list_match   => mf_list_match,
	--  finder_match_addr   => mf_match_address,
	--  current_stream_head => mf_current_address,
	--	literal_valid			  => ctrl_lit_data_valid_flag,
	--  mem_mode            => MEM_MODE,
	--  mem_addr            => MEM_ADDR,
	--  unit_working        => UNIT_WORKING,
	--  matcher_match       => matcher_match,
	--  matcher_match_addr  => matcher_match_addr,
	--	matcher_match_source_addr => matcher_match_source_addr,
	--  finder_done         => mf_length_done,
	--  finder_match_length => mf_match_length,
	--	finder_current_end_stop => mf_current_end_stop,
	--	matcher_rst_pipe 	  => open,
	--	output_match_length => MATCH_LENGTH,
	--	output_literal_length => LIT_LENGTH,
	--	output_match_offset => MATCH_OFFSET,
	--	output_valid => LENGTH_VALID,
	--	pipe_override => PIPE_OVERRIDE,
	--	pipe_override_valid => PIPE_OVERRIDE_VALID
	--);

	LZ4MatcherUnit_i : LZ4MatcherUnit
	generic map(
	   hashMemAddrWidth => 8
	)
	port map (
	  clk        => in_data_axi_aclk,
	  rst        => rst,
	  ce         => matcher_ce,
	  data       => MEM_HASHING_DATA,
	  addr       => MEM_ADDR_DELAY,
	  match      => matcher_match,
	  match_addr => match_addr,
	  match_source_addr => match_source_addr,
		rst_pipe 	 => matcher_rst_pipe
	);

	PPSPacker_match_addr : PPSPacker
	generic map (
	  PORTS      => C_HASHNATOR_COUNT,
	  DATA_WIDTH => LZ4Address'length
	)
	port map (
	  clk         => in_data_axi_aclk,
	  data_in_we  => matcher_match,
	  data_in     => match_addr,
	  data_out    => pps_match_addr,
	  data_out_we => pps_matcher_match
	);

	PPSPacker_match_source_addr : PPSPacker
	generic map (
	  PORTS      => C_HASHNATOR_COUNT,
	  DATA_WIDTH => LZ4Address'length
	)
	port map (
	  clk         => in_data_axi_aclk,
	  data_in_we  => matcher_match,
	  data_in     => match_source_addr,
	  data_out    => pps_match_source_addr,
	  data_out_we => open
	);

	MatchProcessor_i : MatchProcessor
	generic map (
	  C_MATCH_COUNT => C_HASHNATOR_COUNT,
		C_MEMORY_BUS_WIDTH => C_MEMORY_BUS_WIDTH
	)
	port map (
	  clk                  => in_data_axi_aclk,
	  rst                  => rst,
		out_ce 							 => OUTPUT_READY,
		matcher_done				 => DATA_ENDING,
		memory_length 			 => MEM_LENGTH,
	  match_valid          => pps_matcher_match,
	  match_address        => pps_match_addr,
	  match_source_address => pps_match_source_addr,
	  finder_mf_address_1    => MEM_MF_ADDR_1,
	  finder_mf_address_2    => MEM_MF_ADDR_2,
	  finder_mf_data_1       => MEM_MF_DATA_1,
	  finder_mf_data_2       => MEM_MF_DATA_2,
		out_literal_address => mpc_literal_address,
		out_litetal_data_valid => mpc_litetal_data_valid(C_MEMORY_BUS_WIDTH/8-1 downto 0),
		out_literal_length => mpc_literal_length,
		out_match_length => mpc_match_length,
		out_match_offset => mpc_match_offset,
		out_match_valid => mpc_match_valid,
		last_serve => last_serve,
		out_match_ready => M_PROC_READY
	);

	mpc_litetal_data_valid(mpc_litetal_data_valid'length -1 downto C_MEMORY_BUS_WIDTH/8) <= (others => '0');
	MEM_OUT_ADDR <= mpc_literal_address;

	-- User logic ends
	RESET_LOGIC : process(in_data_axi_aclk)
	begin
		if rising_edge(in_data_axi_aclk) then
			if in_data_axi_aresetn = '0' then
				rst <= '1';
			else
				rst <= '0';
				if out_data_axi_tlast = '1' and
					 out_data_axi_tready = '1' then
				  rst <= '1';
			  end if;
		  end if;
		end if;
	end process; -- end RESET_LOGIC
	rstn <= not rst;

	WORK_TIME_CALCULATOR : process(in_data_axi_aclk)
	begin
		if rising_edge(in_data_axi_aclk) then
			if DATA_READY = '1' or in_data_axi_tready = '1' then
				data_clock_register <= data_clock_register + 1;
			end if;
			if DATA_READY = '1' and COMPRESSION_END = '0' then
				work_clock_register <= work_clock_register + 1;
			end if;
		end if;
	end process; -- end WORK_TIME_CALCULATOR
end arch_imp;
