----------------------------------------------------------------------------------
-- Company:
-- Engineer:
--
-- Create Date: 06/06/2018 07:35:00 AM
-- Design Name:
-- Module Name: MatchLengthFinderSim - Behavioral
-- Project Name:
-- Target Devices:
-- Tool Versions:
-- Description:
--
-- Dependencies:
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
----------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.all;
use work.LZ4CompressorLib.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity MatchLengthFinderSim is
--  Port ( );
end MatchLengthFinderSim;

architecture Behavioral of MatchLengthFinderSim is
  constant MEMORY_BUS_WIDTH : natural := 32;

  subtype MEMORY_WORD is std_logic_vector(MEMORY_BUS_WIDTH-1 downto 0);
  type MEMORY_WORD_TYPE is array (0 to 41) of MEMORY_WORD;
  type MEMORY_BYTE_TYPE is array (0 to (MEMORY_WORD_TYPE'length * MEMORY_BUS_WIDTH/8)) of std_logic_vector(7 downto 0);

  signal mem_word : MEMORY_WORD_TYPE := (
    x"01020304", --0
    x"04030201",
    x"01020304",
    x"04030201",
    x"04030201",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000", --9
    x"01020304", --10
    x"04030201",
    x"01020304",
    x"04030201",
    x"04030501",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000", --19
    x"01020304", --20
    x"04030201",
    x"01020304",
    x"04030201",
    x"04030201",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000", --30
    x"01020304", --31
    x"04030201",
    x"01020304",
    x"04030201",
    x"04030501",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000",
    x"00000000" --41
  );

  function init_byte_mem(word_mem : in MEMORY_WORD_TYPE) return MEMORY_BYTE_TYPE is
    variable temp_mem : MEMORY_BYTE_TYPE;
  begin
    for i in 0 to MEMORY_WORD_TYPE'length-1 loop
      for z in 0 to MEMORY_BUS_WIDTH/8-1 loop
        temp_mem(i*MEMORY_BUS_WIDTH/8+z) := word_mem(i)((z+1)*8-1 downto z*8);
      end loop;
    end loop;
    return temp_mem;
  end function;

  function get_word(byte_mem : in MEMORY_BYTE_TYPE; byte_addr : in LZ4Address) return MEMORY_WORD is
    variable tmp_word : MEMORY_WORD;
  begin
    for i in 0 to MEMORY_WORD'length/8-1 loop
      tmp_word((i+1)*8-1 downto i*8) := byte_mem(to_integer(byte_addr) + i);
    end loop;
    return tmp_word;
  end function;

  signal mem : MEMORY_BYTE_TYPE := init_byte_mem(mem_word);

  component MatchLengthFinder
  generic (
    MEMORY_BUS_WIDTH : natural
  );
  port (
    clk             : in  std_logic;
    rst             : in  std_logic;
    list_match      : in  std_logic;
    match_address   : in  LZ4Address;
    current_address : in  LZ4Address;
    mf_address_1    : out LZ4Address;
    mf_address_2    : out LZ4Address;
    mf_data_1       : in  std_logic_vector(MEMORY_BUS_WIDTH-1 downto 0);
    mf_data_2       : in  std_logic_vector(MEMORY_BUS_WIDTH-1 downto 0);
    match_length    : out LZ4MatchLength;
    length_done     : out std_logic
  );
  end component MatchLengthFinder;

  signal clk             : std_logic := '0';
  signal rst             : std_logic := '1';
  signal list_match      : std_logic := '0';
  signal match_address   : LZ4Address := to_unsigned(11*MEMORY_BUS_WIDTH/8, LZ4Address'length);
  signal current_address : LZ4Address := to_unsigned(0, LZ4Address'length);
  signal mf_address_1    : LZ4Address;
  signal mf_address_2    : LZ4Address;
  signal mf_data_1       : std_logic_vector(MEMORY_BUS_WIDTH-1 downto 0);
  signal mf_data_2       : std_logic_vector(MEMORY_BUS_WIDTH-1 downto 0);
  signal match_length    : LZ4MatchLength;
  signal length_done     : std_logic;
begin

  MatchLengthFinder_i : MatchLengthFinder
  generic map (
    MEMORY_BUS_WIDTH => MEMORY_BUS_WIDTH
  )
  port map (
    clk             => clk,
    rst             => rst,
    list_match      => list_match,
    match_address   => match_address,
    current_address => current_address,
    mf_address_1    => mf_address_1,
    mf_address_2    => mf_address_2,
    mf_data_1       => mf_data_1,
    mf_data_2       => mf_data_2,
    match_length    => match_length,
    length_done     => length_done
  );

  clk <= not clk after 10 ns;
  rst <= '0' after 300 ns;
  list_match <= '1' after 353 ns;

  MEM_SIM : process(clk)
  begin
    if rising_edge(clk) then
      mf_data_1 <= get_word(mem, mf_address_1);
      mf_data_2 <= get_word(mem, mf_address_2);
    end if;
  end process; -- end MEM_SIM

end Behavioral;
