library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

package LZ4CompressorLib is
  subtype LZ4Address is unsigned(15 downto 0);
  subtype LZ4MatchLength is unsigned(15 downto 0);

  function HighestOne (i_vector : in std_logic_vector) return natural;
  function HighestZero (i_vector : in std_logic_vector) return natural;
  function Highest (i_vector : in std_logic_vector; i_type : in std_logic) return natural;
  function MAX(LEFT, RIGHT: INTEGER) return INTEGER;
  function MAX(LEFT, RIGHT: unsigned) return unsigned;
  function MAX(LEFT, RIGHT: std_logic_vector) return std_logic_vector;
  function MIN(LEFT, RIGHT: INTEGER) return INTEGER;
  function MIN(LEFT, RIGHT: unsigned) return unsigned;
end package;


package body LZ4CompressorLib is

  function HighestOne (i_vector : in std_logic_vector) return natural is
  begin
    return Highest(i_vector, '1');
  end;

  function HighestZero (i_vector : in std_logic_vector) return natural is
  begin
    return Highest(i_vector, '0');
  end;

  function Highest (i_vector : in std_logic_vector; i_type : in std_logic) return natural is
    variable compare : std_logic_vector(i_vector'length-1 downto 0) := (others => not i_type);
  begin
    for i in 0 to i_vector'length-1 loop
      if(compare = i_vector) then
        return i;
      end if;
      compare(i) := i_type;
    end loop;
    return i_vector'length;
  end;

  function MAX(LEFT, RIGHT: INTEGER) return INTEGER is
  begin
    if LEFT > RIGHT then return LEFT;
    else return RIGHT;
    end if;
  end;

  function MAX(LEFT, RIGHT: unsigned) return unsigned is
  begin
    if LEFT > RIGHT then return LEFT;
    else return RIGHT;
    end if;
  end;

  function MAX(LEFT, RIGHT: std_logic_vector) return std_logic_vector is
  begin
    if unsigned(LEFT) > unsigned(RIGHT) then return LEFT;
    else return RIGHT;
    end if;
  end;


  function MIN(LEFT, RIGHT: INTEGER) return INTEGER is
  begin
    if LEFT < RIGHT then return LEFT;
    else return RIGHT;
    end if;
  end;

  function MIN(LEFT, RIGHT: unsigned) return unsigned is
  begin
    if LEFT < RIGHT then return LEFT;
    else return RIGHT;
    end if;
  end;
end package body;
