library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.all;
use work.LZ4CompressorLib.all;

entity MaxReduction is
  generic (
    INPUT_POWER : natural := 3;
    DATA_WIDTH : natural := 16
  );
  port(
    data_in : in  std_logic_vector(DATA_WIDTH * 2**INPUT_POWER-1 downto 0);
    data_out : out std_logic_vector(DATA_WIDTH-1 downto 0)
  );
end entity; -- end MaxReduction

architecture Behavioral of MaxReduction is
  subtype DATA_TYPE is std_logic_vector(DATA_WIDTH-1 downto 0);
  type TREE_LAYER is array(0 to 2**INPUT_POWER-1) of DATA_TYPE;
  type TREE_INTERCONNECTION is array(0 to INPUT_POWER) of TREE_LAYER;
  signal tree_interconnection_signal : TREE_INTERCONNECTION;
begin

  INPUT_PROCESS : process(all)
  begin
    for i in 0 to 2**INPUT_POWER-1 loop
      tree_interconnection_signal(INPUT_POWER)(i) <= data_in((i+1)*DATA_WIDTH-1 downto i*DATA_WIDTH);
    end loop;
  end process; -- end INPUT_PROCESS
  TREE_GENERATE : for i in INPUT_POWER-1 downto 0 generate
  begin
    COMPARE_GENERATE : for z in 0 to (2**i)-1 generate begin
      tree_interconnection_signal(i)(z) <= MAX(tree_interconnection_signal(i+1)(z*2), tree_interconnection_signal(i+1)((z*2)+1));
    end generate; -- end for generate COMPARE_GENERATE
    tree_interconnection_signal(i)(2**i to 2**INPUT_POWER-1) <= (others => (others => 'Z'));
  end generate; -- end for generate TREE_GENERATE
  data_out <= tree_interconnection_signal(0)(0);
end architecture; -- end Behavioral
