library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.all;

entity PPSPacker is
  generic(
    PORTS : natural := 8;
    DATA_WIDTH : natural := 64
  );
  port(
    clk : in  std_logic;
    data_in_we : in std_logic_vector(PORTS-1 downto 0);
    data_in : in std_logic_vector(DATA_WIDTH*PORTS-1 downto 0);
    data_out : out std_logic_vector(DATA_WIDTH*PORTS-1 downto 0);
    data_out_we : out std_logic_vector(PORTS-1 downto 0)
  );
end entity; -- end PPSMultiplexor

architecture Behavioral of PPSPacker is
  type PPS_ARRAY_T is array(0 to PORTS-1) of natural range 0 to PORTS;
  signal pps_reg : PPS_ARRAY_T;
begin

  PPS_COUNTER : process(all)
    variable tmp : natural;
  begin
    tmp := 0;
    for I in 0 to PORTS-1 loop
      if data_in_we(I) = '1' then
        tmp := tmp + 1;
      end if;
      pps_reg(I) <= tmp;
    end loop;
  end process; -- end PPS_COUNTER

  OUT_MULTIPLEXOR : process(all)
  begin
    data_out_we <= (others => '0');
    data_out <= (others => '0');
    for I in 0 to PORTS-1 loop
      if(pps_reg(I) /= 0 and (I = 0 or pps_reg(I-1) /= pps_reg(I))) then
        data_out(pps_reg(I)*DATA_WIDTH-1 downto (pps_reg(I)-1)*DATA_WIDTH) <= data_in((I+1)*DATA_WIDTH-1 downto I*DATA_WIDTH);
        data_out_we(pps_reg(I)-1) <= '1';
      end if;
    end loop;
  end process; -- end OUT_MULTIPLEXOR
end architecture; -- end Behavioral
